/**********************************************************************************/
/*                            transform.c                                         */
/*                15.03.94 by Sorina COTISEL (cotisel@imag.fr)                    */
/*     Mediatheque IMAG - BP 53X, 38041 GRENOBLE Cedex - mediatheque@imag.fr      */
/*     http://ftpmedia.imag.fr/anteserveur/imag                                   */
/*                                                                                */
/*     Transform functions in HTML or ISO_LATIN form of the edited file           */
/*                                                                                */
/*     corrected errors on 11.04.94 for the xhtmledit-1.2a version                */
/**********************************************************************************/



#include "xhtmledit.h"
#include <fcntl.h>
#include <sys/types.h>
#include <errno.h>

extern int pane_number;
char *tab[255];


/* initialize the ISO_LATIN character table (derived from the iso-8859-1 standard) */

void InitTabChar()
{
   tab[193] = "Aacute;";
   tab[201] = "Eacute;";
   tab[205] = "Iacute;";
   tab[211] = "Oacute;";
   tab[233] = "eacute;";
   tab[224] = "agrave;";
   tab[226] = "acirc;";
   tab[228] = "auml;";
   tab[232] = "egrave;";
   tab[234] = "ecirc;";
   tab[238] = "icirc;";
   tab[239] = "iuml;";
   tab[244] = "ocirc;";
   tab[231] = "ccedil;";
   tab[246] = "ouml;";
   tab[252] = "uuml;";

   tab[218] = "Uacute;";
   tab[221] = "Yacute;";
   tab[225] = "aacute;";
   tab[237] = "iacute;";
   tab[243] = "oacute;";
   tab[250] = "uacute;";
   tab[253] = "yacute;";

   tab[194] = "Acirc;";
   tab[202] = "Ecirc;";
   tab[206] = "Icirc;";
   tab[212] = "Ocirc;";
   tab[219] = "Ucirc;";
   tab[251] = "ucirc;";

   tab[192] = "Agrave;";
   tab[200] = "Egrave;";
   tab[204] = "Igrave;";
   tab[210] = "Ograve;";
   tab[217] = "Ugrave;";
   tab[236] = "igrave;";
   tab[242] = "ograve;";
   tab[249] = "ugrave;";

   tab[199] = "Ccedil;";

   tab[196] = "Auml;";
   tab[203] = "Euml;";
   tab[207] = "Iuml;";
   tab[214] = "Ouml;";
   tab[220] = "Uuml;";
   tab[235] = "euml;";
   tab[255] = "yuml;";

   tab[195] = "Atilde;";
   tab[209] = "Ntilde;";
   tab[213] = "Otilde;";
   tab[227] = "atilde;";
   tab[241] = "ntilde;";
   tab[245] = "otilde;";

   tab[197] = "Aring;";
   tab[229] = "aring;";

   tab[216] = "Oslash;";
   tab[248] = "oslash;";

   tab[198] = "AElig;";
   tab[230] = "aelig;";
   tab[223] = "szlig;";

   tab[208] = "ETH;";
   tab[240] = "eth;";

   tab[222] = "THORN;";
   tab[254] = "thorn;";

   tab[215] = "null;";
   tab[247] = "null;";
}


/* transform the file in HTML or ISO_LATIN form */

char *Transform(file1)
char *file1;
{
   int id1,id2;
   int pid,id;
   char* file2;
   
   file2 = (char *)malloc(30);
   id = getpid();
   sprintf(file2,"/tmp/a2%d",id);

   id1 = open(file1,O_RDONLY,0666);
   if (id1 == -1)
      {
	 perror("Erreur open file1");
	 exit(1);
      }

   id2 = open(file2,O_RDWR|O_CREAT|O_TRUNC,0666);
   if (id2 == -1)
   {
      perror("Erreur open file2");
      exit(1);
   }
   
   if (pane_number == 0)
      TransformHTML(id1,id2);
   else if (pane_number == 1)
      TransformISO_LATIN(id1,id2);
   

   close(id1);
   close(id2);
   return(file2);
}



void TransformHTML(id1,id2)
int id1;
int id2;
{
   unsigned char c,c1;
   int n,i;
   char *str;

   str = (char *)malloc(10);

   while((n = read(id1,&c,1)) != 0)
   {
      for (i = 192; (i<=255) && ( c != i) ;i++);

      if ((i >= 192) && (i <= 255))
      {
         strcpy(str,tab[i]);
         c1 = '&';
         write(id2,&c1,1);
      }
      else
      {
         str[0] = c;
         str[1] = 0;
      }

      write(id2,str,strlen(str));

   } 

   free(str);
} 

 

void TransformISO_LATIN(id1,id2)
int id1;
int id2;
{
   unsigned char c;
   int n,i;
   char ch[10];

   while ((n = read(id1,&c,1)) != 0)
   {
      if (c == '&')
      {  
	 i = 0;
	 do
	 {
	    read(id1,&c,1);
	    ch[i] = c;
	    i++;
	 }
	 while ((c != ';') && (i<10));

	 ch[i] = 0;

	 for (i = 192; (i<=255) && (strcmp(ch,tab[i]) != 0); i++);
	 
	 if (i <= 255)
	 {
	    c = (char)i;
	    write(id2,&c,1);
	 }


	 else
	 {
	    c = '&';
	    write(id2,&c,1);
	    write(id2,ch,strlen(ch));
	 }
      }
      else
	 write(id2,&c,1);
   }
}
